﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PristaniNaMesici
{
    public partial class GameForm : Form
    {
        public GameForm(Game game)
        {
            InitializeComponent();

            this.Game = game;
        }

        public readonly Game Game;

        private void GameForm_Load(object sender, EventArgs e)
        {
            numericUpDown_speed_limit.Value = Game.MaxSpeed;

            checkBox_ai.Checked = Game.AutopilotActive;

            Game.StartAsync();
        }

        private void timer_ticks_Tick(object sender, EventArgs e)
        {
            if (!Game.Started)
                return;

            //Console.WriteLine(Game.CurrentState);

            // Next tick
            {
                if (Game.CurrentState == Game.CurrentStateEnum.InAir)
                {
                    if (checkBox_ai.Checked)
                    {
                        Game.UpdateByAutopilot();
                    }
                    else
                    {
                        Game.UpdateAsync(checkBox_engine_up.Checked, checkBox_engine_right.Checked, checkBox_engine_left.Checked);

                        checkBox_engine_up.Checked = Game.EngineUp;
                        checkBox_engine_left.Checked = Game.EngineLeft;
                        checkBox_engine_right.Checked = Game.EngineRight;
                    }
                }
            }

            // Update visuals
            {
                // Fuel
                {
                    panel_fuel.Invalidate();
                    label_fuel_val.Text = Game.Fuel.ToString();
                }

                // Landing
                panel_landing.Invalidate();

                // Landing distance
                {
                    label_landing_distance_x.Text = "X: " + Game.DistanceToLandingCenterX;
                    label_landing_distance_y.Text = "Y: " + Game.DistanceToGroundYBottom;
                }

                // Landing speed
                {
                    label_speed_x.Text = "X: " + Game.Dx;
                    label_speed_y.Text = "Y: " + Game.Dy;
                }

                // Landing position
                {
                    label_position_x.Text = "X: " + Game.X;
                    label_position_y.Text = "Y: " + Game.Y;
                }
            }
        }

        private void panel_fuel_Paint(object sender, PaintEventArgs e)
        {
            var g = e.Graphics;

            float val = Game.MaxFuel == 0 ? 0 : Game.Fuel / Game.MaxFuel;
            float height = val * panel_fuel.ClientSize.Height;

            g.FillRectangle(Game.EnoughFuelForSlowdown ? Brushes.Green : Brushes.Red, 0, panel_fuel.ClientSize.Height - height, panel_fuel.ClientSize.Width, height);
        }

        private void GameForm_KeyDown(object sender, KeyEventArgs e)
        {
            var key = e.KeyCode;

            // Hold control
            {
                if (key == Keys.W || key == Keys.Up)
                    checkBox_engine_up.Checked = false;

                if (key == Keys.D || key == Keys.Right)
                    checkBox_engine_right.Checked = false;

                if (key == Keys.A || key == Keys.Left)
                    checkBox_engine_left.Checked = false;
            }
        }

        private void GameForm_KeyUp(object sender, KeyEventArgs e)
        {
            var key = e.KeyCode;

            // Hold control
            {
                if (key == Keys.W || key == Keys.Up)
                    checkBox_engine_up.Checked = false;

                if (key == Keys.D || key == Keys.Right)
                    checkBox_engine_right.Checked = false;

                if (key == Keys.A || key == Keys.Left)
                    checkBox_engine_left.Checked = false;
            }

            // Toggle control
            {
                if (key == Keys.I)
                    checkBox_engine_up.Checked = !checkBox_engine_up.Checked;

                if (key == Keys.L)
                    checkBox_engine_right.Checked = !checkBox_engine_right.Checked;

                if (key == Keys.J)
                    checkBox_engine_left.Checked = !checkBox_engine_left.Checked;
            }
        }

        public static readonly Image LanderImage = Image.FromFile("lunar_lander.png");

        private void panel_landing_Paint(object sender, PaintEventArgs e)
        {
            var g = e.Graphics;

            // Scale (render setup)
            {
                // Presunout spodni hranu na 0 a Y+ smerem nahoru
                // Default: 
                //  [0,0] = levy hori roh
                //  X+ vpravo
                //  Y+ dolu
                g.TranslateTransform(0, panel_landing.ClientSize.Height);// posun na spodni hranu
                g.ScaleTransform(1, -1);// prevraceni Y (Y+ nahoru)
                
                if(checkBox_zoom.Checked)
                // Zoom to lander (lander on upper half and landing on bottom) and center landing
                {
                    // Center landing point
                    {
                        g.TranslateTransform(Game.LandingCenter, 0);
                    }
                    // Change height
                    {
                        float curr_width = panel_landing.ClientSize.Width;
                        float curr_height = panel_landing.ClientSize.Height;

                        // prepare ration
                        double ratio_wh = ((double)curr_width) / curr_height;
                        double ratio_hw = ((double)curr_height) / curr_width;

                        float target_width_w = 2 * (Game.DistanceToLandingCenterX); // define target width
                        float target_height_w = (float)(ratio_hw * target_width_w); // calculate height while keeping ration

                        float target_height_h = Game.Y + Game.SizeTotalY; // define target height
                        float target_width_h = (float)(ratio_wh * target_height_h); // calculate width while keeping ration

                        bool use_w = target_height_h <= target_height_w; // Decide which scale to use (which fits better)

                        // Convert to scale
                        float scale_width = (use_w ? target_width_w : target_width_h) / curr_width;
                        float scale_height = (use_w ? target_height_w : target_height_h) / curr_height;

                        g.ScaleTransform(1/scale_width, 1/scale_height);
                    }
                }
                else
                // 0-200 na obou osach
                {
                    float curr_width = panel_landing.ClientSize.Width;
                    float curr_height = panel_landing.ClientSize.Height;

                    float scale_width = 200 / curr_width;
                    float scale_height = 200 / curr_height;

                    g.ScaleTransform(1/scale_width, 1/scale_height);
                }
            }

            // Landing
            {
                // Pens
                var pen_ok = Pens.Green;
                var pen_warning = Pens.Gray;
                var pen_fail = Pens.Red;

                // Height lines
                if (checkBox_height_lines.Checked)
                {
                    // ZOOM ON: End of navigation lines with top edge of lander
                    // ZOOM OFF: Upper edge of "screen"
                    var y_limit = checkBox_zoom.Checked ? Game.Y + Game.SizeTotalY * 2 : 200;
                    var increment = (int)numericUpDown_height_lines.Value;

                    // ZOOM ON: Big distance (out of "screen")
                    // ZOOM OFF: From left to right edge
                    var sizeMin = checkBox_zoom.Checked ? -1000 : 0;
                    var sizeMax = checkBox_zoom.Checked ? 1000 : 200;

                    // draw needed number of lines
                    for (int hline_y = increment; hline_y <= y_limit; hline_y += increment)
                        g.DrawLine(Pens.LightGray, sizeMin, hline_y, sizeMax, hline_y);
                }

                // Navigation lines
                if (checkBox_navigation_lines.Checked)
                {
                    // ZOOM ON: End of navigation lines with top edge of lander
                    // ZOOM OFF: Upper edge of "screen"
                    var y_limit = checkBox_zoom.Checked ? Game.Y + Game.SizeTotalY : 200;

                    // Left landing edge
                    g.DrawLine(pen_warning, Game.LandingLeft, 0, Game.LandingLeft, y_limit);
                    // Left edge of safe landing
                    g.DrawLine(pen_ok, Game.LandingLeftCenter, 0, Game.LandingLeftCenter, y_limit);

                    // Right landing edge
                    g.DrawLine(pen_warning, Game.LandingRight, 0, Game.LandingRight, y_limit);
                    // Right edge of safe landing
                    g.DrawLine(pen_ok, Game.LandingRightCenter, 0, Game.LandingRightCenter, y_limit);
                }

                // Ground
                {
                    // ZOOM ON: Big distance (out of "screen")
                    // ZOOM OFF: From left to right edge
                    var sizeMin = checkBox_zoom.Checked ? -1000 : 0;
                    var sizeMax = checkBox_zoom.Checked ? 1000 : 200;

                    // Left from landing
                    g.DrawLine(pen_fail, sizeMin, 0, Game.LandingLeft, 0);
                    // Left landing edge
                    g.DrawLine(pen_warning, Game.LandingLeft, 0, Game.LandingLeftCenter, 0);

                    // Landing area
                    g.DrawLine(pen_ok, Game.LandingLeftCenter, 0, Game.LandingRightCenter, 0);

                    // Right landing edge
                    g.DrawLine(pen_warning, Game.LandingRightCenter, 0, Game.LandingRight, 0);
                    // Right from landing
                    g.DrawLine(pen_fail, Game.LandingRight, 0, sizeMax, 0);
                }
            }

            // Lander image scaled to 20x20
            g.DrawImage(LanderImage, Game.X - Game.SizeFromCenterX, Game.Y - Game.SizeFromCenterY, Game.SizeTotalX, Game.SizeTotalY);
        }

        private void checkBox_ai_CheckedChanged(object sender, EventArgs e)
        {
            Game.AutopilotActive = checkBox_ai.Checked;
        }
    }
}
